"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const constant_1 = require("../../constant");
const common_1 = require("../../plugin/common");
const utils_1 = require("../../utils");
const decorator_1 = require("../decorator");
const config_file_meta_1 = require("../utils/config_file_meta");
const config_1 = tslib_1.__importDefault(require("./config"));
let PluginConfigLoader = class PluginConfigLoader extends config_1.default {
    static async is(opts) {
        if (this.isConfigDir(opts)) {
            return (0, utils_1.isMatch)(opts.filename, constant_1.PLUGIN_CONFIG_PATTERN);
        }
        return false;
    }
    async load(item) {
        const { env } = (0, config_file_meta_1.getConfigMetaFromFilename)(item.filename);
        const configObj = await this.loadConfigFile(item);
        for (const pluginName of Object.keys(configObj)) {
            const pluginConfigItem = configObj[pluginName];
            if (pluginConfigItem.package) {
                // convert package to path when load plugin config
                if (pluginConfigItem.path) {
                    throw new Error(`Plugin ${pluginName} config can't have both package and path at ${item.path}`);
                }
                const loaderState = item.loaderState;
                pluginConfigItem.path = (0, common_1.getPackagePath)(pluginConfigItem.package, [loaderState === null || loaderState === void 0 ? void 0 : loaderState.baseDir]);
                delete pluginConfigItem.package;
                configObj[pluginName] = pluginConfigItem;
            }
        }
        this.configurationHandler.setConfig(env, {
            plugin: configObj,
        });
        return configObj;
    }
};
PluginConfigLoader = tslib_1.__decorate([
    (0, decorator_1.DefineLoader)('plugin-config')
], PluginConfigLoader);
exports.default = PluginConfigLoader;
