"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PluginFactory = void 0;
const common_1 = require("./common");
const impl_1 = require("./impl");
class PluginFactory {
    static async create(name, item, opts) {
        const pluginInstance = new impl_1.Plugin(name, item, opts);
        await pluginInstance.init();
        return pluginInstance;
    }
    static async createFromConfig(config, opts) {
        const pluginInstanceMap = new Map();
        for (const [name, item] of Object.entries(config)) {
            const pluginInstance = await PluginFactory.create(name, item, opts);
            if (pluginInstance.enable) {
                pluginInstanceMap.set(name, pluginInstance);
            }
        }
        let pluginDepEdgeList = [];
        // Topological sort plugins
        for (const [_name, pluginInstance] of pluginInstanceMap) {
            pluginInstance.checkDepExisted(pluginInstanceMap);
            pluginDepEdgeList = pluginDepEdgeList.concat(pluginInstance.getDepEdgeList());
        }
        const pluginSortResult = (0, common_1.topologicalSort)(pluginInstanceMap, pluginDepEdgeList);
        if (pluginSortResult.length !== pluginInstanceMap.size) {
            const diffPlugin = [...pluginInstanceMap.keys()].filter(name => !pluginSortResult.includes(name));
            throw new Error(`There is a cycle in the dependencies, wrong plugin is ${diffPlugin.join(',')}.`);
        }
        return pluginSortResult.map(name => pluginInstanceMap.get(name));
    }
}
exports.PluginFactory = PluginFactory;
